__name__ = "mwoauth"
__version__ = "0.3.7"
__author__ = "Aaron Halfaker / Filippo Valsorda"
__author_email__ = "aaron.halfaker@gmail.com"
__description__ = "A generic MediaWiki OAuth handshake helper."
__license__ = "MIT"
__url__ = "https://github.com/mediawiki-utilities/python-mwoauth"

all = [__name__, __version__, __author__, __author_email__, __description__,
       __license__, __url__]
